import os
import shutil
import tempfile
import unittest
from unittest import mock

from .. import apache

from charmhelpers.core import unitdata


class ApacheTestCase(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        patcher = mock.patch('charmhelpers.core.hookenv.config')
        self.mock_config = patcher.start()
        self.addCleanup(patcher.stop)

        self.tmpdir = tempfile.mkdtemp(prefix='charm-unittests-')
        self.addCleanup(shutil.rmtree, self.tmpdir)
        os.environ['UNIT_STATE_DB'] = os.path.join(self.tmpdir, '.unit-state.db')
        unitdata.kv().set('squid-cache-disk', 5000)
        unitdata.kv().set('squid-disk-caches', [('/srv', 2000)])

    @mock.patch('multiprocessing.cpu_count')
    def test_tune_mpm_configs(self, cpu_count):

        cpu_count.return_value = 4
        config = {
            'startservers': 0,
            'minsparethreads': 0,
            'maxsparethreads': 0,
            'threadlimit': 0,
            'threadsperchild': 0,
            'serverlimit': 0,
            'maxrequestworkers': 0,
            'maxconnectionsperchild': 10000,
        }
        expected = {
            'startservers': 1,
            'minsparethreads': 1024,
            'maxsparethreads': 2048,
            'threadlimit': 1024,
            'threadsperchild': 1024,
            'serverlimit': 2,
            'maxrequestworkers': 2048,
            'maxconnectionsperchild': 10000,
        }
        self.assertEqual(apache.tune_mpm_configs(config), expected)

        cpu_count.return_value = 40
        config = {
            'startservers': 0,
            'minsparethreads': 0,
            'maxsparethreads': 0,
            'threadlimit': 0,
            'threadsperchild': 0,
            'serverlimit': 0,
            'maxrequestworkers': 0,
            'maxconnectionsperchild': 10000,
        }
        expected = {
            'startservers': 1,
            'minsparethreads': 1280,
            'maxsparethreads': 2560,
            'threadlimit': 1280,
            'threadsperchild': 1280,
            'serverlimit': 2,
            'maxrequestworkers': 2560,
            'maxconnectionsperchild': 10000,
        }
        self.assertEqual(apache.tune_mpm_configs(config), expected)

        cpu_count.return_value = 1
        config = {
            'startservers': 0,
            'minsparethreads': 0,
            'maxsparethreads': 0,
            'threadlimit': 0,
            'threadsperchild': 0,
            'serverlimit': 0,
            'maxrequestworkers': 0,
            'maxconnectionsperchild': 10000,
        }
        expected = {
            'startservers': 1,
            'minsparethreads': 256,
            'maxsparethreads': 512,
            'threadlimit': 256,
            'threadsperchild': 256,
            'serverlimit': 2,
            'maxrequestworkers': 512,
            'maxconnectionsperchild': 10000,
        }
        self.assertEqual(apache.tune_mpm_configs(config), expected)

        cpu_count.return_value = 2
        config = {
            'startservers': 0,
            'minsparethreads': 0,
            'maxsparethreads': 0,
            'threadlimit': 0,
            'threadsperchild': 0,
            'serverlimit': 0,
            'maxrequestworkers': 0,
            'maxconnectionsperchild': 10000,
        }
        expected = {
            'startservers': 1,
            'minsparethreads': 512,
            'maxsparethreads': 1024,
            'threadlimit': 512,
            'threadsperchild': 512,
            'serverlimit': 2,
            'maxrequestworkers': 1024,
            'maxconnectionsperchild': 10000,
        }
        self.assertEqual(apache.tune_mpm_configs(config), expected)

        cpu_count.return_value = 4
        config = {
            'startservers': 5,
            'minsparethreads': 10,
            'maxsparethreads': 100,
            'threadlimit': 32,
            'threadsperchild': 32,
            'serverlimit': 10,
            'maxrequestworkers': 8192,
            'maxconnectionsperchild': 5000,
        }
        expected = {
            'startservers': 5,
            'minsparethreads': 10,
            'maxsparethreads': 100,
            'threadlimit': 32,
            'threadsperchild': 32,
            'serverlimit': 10,
            'maxrequestworkers': 8192,
            'maxconnectionsperchild': 5000,
        }
        self.assertEqual(apache.tune_mpm_configs(config), expected)
