import os
import shutil
import sys
import tempfile
import unittest
from unittest import mock

# We also need to mock up charms.layer so we can run unit tests without having
# to build the charm and pull in layers such as layer-status.
sys.modules['charms.layer'] = mock.MagicMock()

from charms.layer import status  # NOQA: E402
from charmhelpers.core import unitdata  # NOQA: E402

# Add path to where our reactive layer lives and import.
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))
from reactive import ubuntu_repository_cache  # NOQA: E402


class TestCharm(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.tmpdir = tempfile.mkdtemp(prefix='charm-unittests-')
        self.addCleanup(shutil.rmtree, self.tmpdir)
        os.environ['UNIT_STATE_DB'] = os.path.join(self.tmpdir, '.unit-state.db')

        unitdata.kv().set('test', {})

        self.charm_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

        patcher = mock.patch('charmhelpers.core.hookenv.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_log.return_value = ''

        patcher = mock.patch('charmhelpers.core.hookenv.charm_dir')
        self.mock_charm_dir = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_charm_dir.return_value = self.charm_dir

        patcher = mock.patch('charmhelpers.core.hookenv.local_unit')
        self.mock_local_unit = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_local_unit.return_value = 'mock-ubuntu-repository-cache/0'

        patcher = mock.patch('charmhelpers.core.hookenv.config')
        self.mock_config = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_config.return_value = {'nagios_context': 'juju'}

        patcher = mock.patch('charmhelpers.core.host.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_log.return_value = ''

        status.active.reset_mock()
        status.blocked.reset_mock()
        status.maintenance.reset_mock()
        # We also need to reset config.changed states.
        if os.path.exists('.juju-persistent-config'):
            os.unlink('.juju-persistent-config')

    @mock.patch('charms.reactive.set_flag')
    def test_set_active(self, set_flag):
        ubuntu_repository_cache.set_active()
        status.active.assert_called_once_with('Ready')
        set_flag.assert_called_once_with('ubuntu-repository-cache.active')

        status.active.reset_mock()
        # bzr - "revision-id: haw.loeung@canonical.com-20210131223324-uax4glwgttv0wftp"
        ubuntu_repository_cache.set_active(os.path.join(self.charm_dir, 'tests/unit/files/version'))
        status.active.assert_called_once_with('Ready (source version/commit 20210131-uax4glwg)')

        status.active.reset_mock()
        # git - "uax4glw"
        ubuntu_repository_cache.set_active(os.path.join(self.charm_dir, 'tests/unit/files/version2'))
        status.active.assert_called_once_with('Ready (source version/commit uax4glw)')

        status.active.reset_mock()
        # bzr - "haw.loeung@canonical.com-20210131233021-66dfilb1615jppxt"
        ubuntu_repository_cache.set_active(os.path.join(self.charm_dir, 'tests/unit/files/version3'))
        status.active.assert_called_once_with('Ready (source version/commit 20210131-66dfilb1)')

        status.active.reset_mock()
        ubuntu_repository_cache.set_active(os.path.join(self.charm_dir, 'tests/unit/files/version4'))
        status.active.assert_called_once_with('Ready (source version/commit somerand)')
        status.active.reset_mock()
        ubuntu_repository_cache.set_active(os.path.join(self.charm_dir, 'tests/unit/files/version5'))
        status.active.assert_called_once_with('Ready (source version/commit commit-id)')
